#!/bin/bash

if [ -z "$1" ]; then
    printf " $ %s [amsbackend|nginx]\n" $0
    exit 1
fi

[ -z "${DOCKER}" ] && DOCKER=$(which docker)
[ -z "${DOCKER}" ] && DOCKER=/usr/bin/docker
[ -z "${AMS_BACKEND_DIR}" ] && AMS_BACKEND_DIR=`dirname $(find /home/astros -name run.sh)`

if [ "$1" = "amsbackend" ]; then
    # Run Docker Image (amsbackend)
    $DOCKER run \
        -v $AMS_BACKEND_DIR/.storage:/usr/src/app/storage \
        -v $AMS_BACKEND_DIR/../guest:/usr/src/app/external \
        -v $AMS_BACKEND_DIR/../build.major:/usr/src/app/build.major \
        -v $AMS_BACKEND_DIR/../build.minor:/usr/src/app/build.minor \
        -v $AMS_BACKEND_DIR/../build.revision:/usr/src/app/build.revision \
        -v $AMS_BACKEND_DIR/../build.buildnumber:/usr/src/app/build.buildnumber \
        -h AstrogateMGM \
        -d --rm -p 5688:5688 \
        --name amsbackend amsbackend:latest /bin/bash /usr/src/app/external/AMS
    exit $?
fi

if [ "$1" = "nginx" ]; then
    # Run Docker Image (nginx)
    $DOCKER run \
        -v $AMS_BACKEND_DIR/../nginx/default.conf:/etc/nginx/conf.d/default.conf \
        -v $AMS_BACKEND_DIR/../nginx/dist:/usr/share/nginx/html \
        -v $AMS_BACKEND_DIR/../nginx/nginx.crt:/etc/nginx/ssl/nginx.crt \
        -v $AMS_BACKEND_DIR/../nginx/nginx.key:/etc/nginx/ssl/nginx.key \
        --name nginx \
        -d --rm -p 443:443 -p 80:80 nginx
    exit $?
fi
