#!/bin/bash

# Used: watchdog.sh autoconf.sh

[ -z "${NODE}" ] && NODE=/usr/bin/node
[ -z "${LOG_PATH}" ] && LOG_PATH=/home/astros/log
[ -z "${UPDATE_PATH}" ] && UPDATE_PATH=/home/astros/update
[ -z "${UPDATING_PATH}" ] && UPDATING_PATH=/home/astros/updating

zipname=

if [ "$1" = "autoclean" ]; then
    [ -d $UPDATING_PATH ] && rm -rf $UPDATING_PATH && printf " * [OK] Removed $UPDATING_PATH\n" >> $LOG_PATH
    exit 0
fi

check_busy () {

    printf " * Starting ASTROS Manager Service Updating\n" >> $LOG_PATH

    if [ -d $UPDATING_PATH ]; then
        printf " * $UPDATING_PATH existed. Do nothing to prevent reentrancy\n" >> $LOG_PATH
        true && return
    else
        false && return
    fi
}

unpack () {

    [ ! -f $UPDATE_PATH/*zip ] && false && return

    [ ! $(/usr/bin/ls -l $UPDATE_PATH/*zip | wc -l) -ne 1 ] && false && return

    /usr/bin/unzip $UPDATE_PATH/*zip -d $UPDATING_PATH
    zipname=$(/usr/bin/ls $UPDATE_PATH/*zip | /usr/bin/xargs -n 1 basename)

    # forward compatible for encrypted archive
    ret=$(find $UPDATING_PATH -name *.encrypted | wc -l)
    if [ $ret -eq 1 ]; then
        mkdir -p $UPDATING_PATH/.stash
        $NODE $(find /home/astros -name amscrypto*) dec $(find $UPDATING_PATH -name *.encrypted)
        $NODE $(find /home/astros -name amscrypto*) extract $(find $UPDATING_PATH -name *.decrypted) -o $UPDATING_PATH/.stash
        rootdir=`dirname $(find $UPDATING_PATH/.stash -name host)`
        mv $rootdir $UPDATING_PATH
        rm -rf $UPDATING_PATH/.stash
        rm -rf `dirname $(find $UPDATING_PATH -name *.decrypted)`
    fi

    [ -d $UPDATING_PATH ] && /usr/bin/mv $UPDATING_PATH/* $UPDATING_PATH/AMS_SERVICE_CURRENT
    [ -d $UPDATING_PATH/AMS_SERVICE_CURRENT ] && true || false
}

start_update () {
    # check AMS version (older, newer or same)
    time_current=0
    time_unknown=0
    [ -f $UPDATING_PATH/AMS_SERVICE_CURRENT/build.timestamp ] \
        && time_unknown=$(< $UPDATING_PATH/AMS_SERVICE_CURRENT/build.timestamp) \
        || time_current=-1

    [ -f /home/astros/AMS_SERVICE_CURRENT/build.timestamp ] \
        && time_current=$(< /home/astros/AMS_SERVICE_CURRENT/build.timestamp) \
        || time_current=-1

    if [ $time_unknown -eq $time_current ]; then
        printf " * Found new ASTROS Manager Service Package. (%s)\n * Do nothing due to the package you provided is the same as the current version.\n" $zipname >> $LOG_PATH
        /usr/bin/mv $UPDATE_PATH/*zip $UPDATE_PATH/same_version
        [ -d $UPDATING_PATH ] && rm -rf $UPDATING_PATH
        false && return
    else
        printf "\nFound new ASTROS Manager Service Package. (%s)\n" $zipname > /dev/tty1
        printf " * Found new ASTROS Manager Service Package. (%s)\n" $zipname >> $LOG_PATH

        [ -d /home/astros/AMS_SERVICE_CURRENT/host/.storage ] \
            && /usr/bin/cp -rp /home/astros/AMS_SERVICE_CURRENT/host/.storage $UPDATING_PATH/AMS_SERVICE_CURRENT/host/

        /usr/bin/rm -rf /home/astros/AMS_SERVICE_CURRENT \
            && /usr/bin/mv $UPDATING_PATH/AMS_SERVICE_CURRENT /home/astros/AMS_SERVICE_CURRENT

        if [ $time_unknown -gt $time_current ]; then
            /usr/bin/mv $UPDATE_PATH/*zip $UPDATE_PATH/upgrade_succeeded
            printf "\nSucceeded to upgrade ASTROS Manager Service.\n" > /dev/tty1
            printf " * Succeeded to upgrade ASTROSManager Service. (%s)\n" $zipname >> $LOG_PATH
        else
            /usr/bin/mv $UPDATE_PATH/*zip $UPDATE_PATH/downgrade_succeeded
            printf "\nSucceeded to downgrade ASTROS Manager Service.\n" > /dev/tty1
            printf " * Succeeded to downgrade ASTROSManager Service. (%s)\n" $zipname >> $LOG_PATH
        fi

        [ -d $UPDATING_PATH ] && rm -rf $UPDATING_PATH

        true && return
    fi
}

# replace AMS_SERVICE_CURRENT with new version
if [ -d $UPDATE_PATH ]; then

    check_busy && exit 1

    unpack || exit 1

    if start_update; then
        printf "\nThe system will reboot after a few seconds ...\n" > /dev/tty1
        #$DOCKER kill $($DOCKER ps -q) && printf "\t." > /dev/tty1
        #$DOCKER rm $($DOCKER ps -a -q) && printf "." > /dev/tty1
        #$DOCKER rmi $($DOCKER images -q) && printf ".\n" > /dev/tty1

        cd $(dirname "$(find /home/astros -name autoconf.sh)") \
            && /bin/bash $(find /home/astros -name autoconf.sh) >> $LOG_PATH 2>&1 \
            && sync

        printf "\t3\n" > /dev/tty1 && /usr/bin/sleep 1
        printf "\t2\n" > /dev/tty1 && /usr/bin/sleep 1
        printf "\t1\n" > /dev/tty1 && /usr/bin/sleep 2

        #/usr/sbin/shutdown -r now
        exit 0 # 0 means WILL reboot
    else
        exit 1 # 1 means NOT reboot
    fi
fi
