#!/bin/bash

LOG_PATH=/home/astros/log
VM_UPDATE_PATH=/home/astros/upgrade

LOCK () {
    [ -f /home/astros/watchdog.lock ] && exit 1 || /usr/bin/touch /home/astros/watchdog.lock && printf " * LOCK\n" >> $LOG_PATH
}

UNLOCK () {
    [ -f /home/astros/watchdog.lock ] && rm /home/astros/watchdog.lock && printf " * UNLOCK\n" >> $LOG_PATH
}

generate_nginx_conf () {
    IP=$(/bin/hostname -I | awk '{ print $1 }')
    cat $AMS_BACKEND_DIR/../nginx/https.conf > $AMS_BACKEND_DIR/../nginx/default.conf
    echo 'upstream backend { server '$IP':5688; }' >> $AMS_BACKEND_DIR/../nginx/default.conf
}

# move log to shared folder
[ $(/usr/bin/wc -l $LOG_PATH | /usr/bin/awk '{print $1}') -gt 1000 ] && /usr/bin/tail -n 200 $LOG_PATH > $LOG_PATH.tail && mv $LOG_PATH.tail $LOG_PATH
if [ -d /home/astros/debug ]; then
    [ -f /home/astros/log ] && var1=$(/usr/bin/md5sum $LOG_PATH | awk '{print $1}')
    [ -f /home/astros/debug/log ] && var2=$(/usr/bin/md5sum /home/astros/debug/log | awk '{print $1}')
    [ "$var1" != "$var2" ] && /usr/bin/cp /home/astros/log /home/astros/debug/
fi
[ -d /home/astros/debug ] && [ ! -f /home/astros/debug/log ] && [ -f /home/astros/log ] && /usr/bin/cp /home/astros/log /home/astros/debug/

# Update ASTROS Manager service with new version
if [ -f $VM_UPDATE_PATH/*zip ]; then
    if [ ! -d /home/astros/update ]; then
        mkdir /home/astros/update
        /usr/bin/mv $VM_UPDATE_PATH/*zip /home/astros/update/
        /bin/bash $AMS_BACKEND_DIR/update.sh >> $LOG_PATH 2>&1
        ret=$?
        /usr/bin/mv /home/astros/update/* $VM_UPDATE_PATH
        rm -rf /home/astros/update
        sync
        [ $ret -eq 0 ] &&  /usr/sbin/shutdown -r now
    fi
fi

# prevent virtual machine IP address conflict issue
test -e $AMS_BACKEND_DIR/.storage/machine-inited
[ $? != 0 ] && rm /etc/machine-id && systemd-machine-id-setup && touch $AMS_BACKEND_DIR/.storage/machine-inited

# generate configuration for NGINX
test -e $AMS_BACKEND_DIR/../nginx/default.conf
[ $? != 0 ] && generate_nginx_conf

# check if mac address of network adapter changed
/bin/bash $(find /home/astros -name check.sh)

/usr/sbin/ip addr show docker0 | grep $(hostname -I | awk '{print $1}') > /dev/null
ret=$?
[ $ret -eq 0 ] && printf " - Invaild network adapter\n - please change networking mode to bridged with physical network adapter\n" >> $LOG_PATH
[ $ret -eq 0 ] && printf "\nInvaild network adapter, please change networking mode to bridged with physical network adapter\n" > /dev/tty1 && exit 0

pgrep -f hostclient
if [ $? -eq 1 ]; then
    UNLOCK
    cd $AMS_BACKEND_DIR
    printf "## %s\n" $(/usr/bin/date) >> $LOG_PATH
    printf " $ node hostclient.bundle.js\n" >> $LOG_PATH
    $NODE hostclient.bundle.js >> $LOG_PATH 2>&1 &
    sleep 1
    #/usr/bin/clear > /dev/tty1
fi

LOCK

# Build Docker Image (amsbackend)
ret=$($DOCKER images amsbackend:latest -q | wc -l)
if [ $ret -eq 0 ]; then
    printf " * Build Docker Image (amsbackend)\n" >> $LOG_PATH
    /bin/bash $(find /home/astros -name build.sh) amsbackend > /dev/tty1 2>&1
    [ $? -ne 0 ] printf " * FAILED to build docker image (amsbackend)\n" >> $LOG_PATH
fi

# Run Docker Image (amsbackend)
ret=$($DOCKER ps --filter name=amsbackend | wc -l)
if [ $ret -eq 1 ]; then
    [ ! -d "$AMS_BACKEND_DIR/.storage" ] && mkdir $AMS_BACKEND_DIR/.storage -p
    printf "\nASTROS Manager service is starting (backend) ...\n" > /dev/tty1
    printf " * Run Docker Image (amsbackend)\n" >> $LOG_PATH
    /bin/bash $(find /home/astros -name run.sh) amsbackend > /dev/tty1 2>&1
    ret=$?
    [ $ret -eq 0 ] \
        && printf " * SUCCEEDED to run docker image (amsbackend)\n" >> $LOG_PATH \
        || printf " * FAILED to run docker image (amsbackend)\n" >> $LOG_PATH
fi

# check if ip address changed
[ ! -f $AMS_BACKEND_DIR/.tmp.address ] && /bin/hostname -I | awk '{print $1}' > $AMS_BACKEND_DIR/.tmp.address
if [ "$(/usr/bin/cat $AMS_BACKEND_DIR/.tmp.address)" != "$(/bin/hostname -I | awk '{print $1}')" ]; then
    /bin/hostname -I | awk '{print $1}' > $AMS_BACKEND_DIR/.tmp.address
    [ $($DOCKER ps -q --filter name=nginx | wc -l) -gt 0 ] && \
        printf " * Stop nginx due to ip address changed\n" >> $LOG_PATH && \
        $DOCKER stop $($DOCKER ps -q --filter name=nginx)
    #$DOCKER rmi $($DOCKER images nginx -q) && printf " * Remove nginx due to ip address changed\n"
    generate_nginx_conf
fi

# Build Docker Image (nginx)
ret=$($DOCKER images nginx:latest -q | wc -l)
if [ $ret -ne 1 ]; then
    printf " * Build Docker Image (nginx)\n" >> $LOG_PATH
    /bin/bash $(find /home/astros -name build.sh) nginx > /dev/tty1 2>&1
    [ $? -ne 0 ] printf " * FAILED to build docker image (nginx)\n" >> $LOG_PATH
fi

# Run Docker Image (nginx)
ret=$($DOCKER ps --filter name=nginx | wc -l)
if [ $ret -eq 1 ]; then
    printf " * Run Docker Image (nginx)\n" >> $LOG_PATH
    printf "\nASTROS Manager service is starting (nginx) ...\n" > /dev/tty1
    /bin/bash $(find /home/astros -name run.sh) nginx > /dev/tty1 2>&1
    ret=$?
    [ $ret -eq 0 ] \
        && printf " * SUCCEEDED to run docker image(nginx)\n" >> $LOG_PATH \
        || printf " * FAILED to run docker image(nginx)\n" >> $LOG_PATH

    [ $ret -eq 0 ] && printf " - ASTROS Manager service is ready now, IP address: %s, hostname: %s\n" $(hostname -I | awk '{ print $1 }') $(hostname) >> $LOG_PATH
    /bin/bash $AMS_BACKEND_DIR/refresh.sh
fi

UNLOCK

# Remove all Exited containers
$DOCKER ps -a | grep exited -i | cut -d ' ' -f 1 | xargs $DOCKER rm